@@ -231,13 +231,15 @@ def get_latest_version():
 def call_ctypesgen(target_dir, include_dir):
     
     bindings = join(target_dir, BindingsFileName)
+    headers = sorted(glob( join(include_dir, "*.h") ))
     
-    ctypesgen_cmd = ["ctypesgen", "--library", "pdfium", "--strip-build-path", target_dir, "-L", "."] + sorted(glob( join(include_dir, "*.h") )) + ["-o", bindings]
+    ctypesgen_cmd = ["ctypesgen", "--library", "pdfium", "--runtime-libdir", ".", "--strip-build-path=%s" % include_dir, *headers, "-o", bindings]
     run_cmd(ctypesgen_cmd, cwd=target_dir)
     
+    # --strip-build-path fails for the preamble: https://github.com/ctypesgen/ctypesgen/issues/160
     with open(bindings, "r") as file_reader:
         text = file_reader.read()
-        text = text.replace(target_dir, ".")
+        text = text.replace(include_dir, ".")
         text = text.replace(HomeDir, "~")
     
     with open(bindings, "w") as file_writer:
